from utils import load_data_with_error, one_hot
from utils import create_model_lstm, BatchGenerator

from tensorflow.keras.optimizers import Adam
from tensorflow.keras.callbacks import EarlyStopping, ModelCheckpoint



def main():
    # Change the following parameters to achieve different training outcomes
    tps = [3,4,5,6,7,8,9,10,15,20]          #List of time point lengths to be used for training
    error_range = [0,0.5,1, 2]              #List of standard deviations to be used when adding Gaussian error to the data
    epochs = 3000                           #Max number of epochs to train for
    start = 0                               
    batch_size_training = 2048              #Reduce batch size if memory problems are encountered when training

    #Load the data
    x1_train, x2_train, y_train, x1_val, x2_val, y_val, x1_test, x2_test, y_test = load_data_with_error()
    y_train_oh = one_hot(y_train)
    y_val_oh = one_hot(y_val)
    print('Training set size:  ',x1_train.shape, x2_train.shape, y_train.shape)
    print('Validation set size:',x1_val.shape, x2_val.shape, y_val.shape)
    
    #Create the model graph
    model = create_model_lstm(input1_shape = x1_train.shape[1:3], input2_shape = (None,x2_train[20][0].shape[-1]), output_shape = y_train_oh.shape)
    opt = Adam(learning_rate = 0.0001)
    model.compile(optimizer = opt, loss = 'categorical_crossentropy', metrics = ['categorical_accuracy'])
    
    early_stopping = EarlyStopping(
        monitor='val_categorical_accuracy', 
        verbose=1,
        patience=300,
        mode='max',
        restore_best_weights=True)
    checkpoint = ModelCheckpoint("Data/best_model_weights", monitor = 'val_categorical_accuracy', verbose = 1, save_best_only = True, save_weights_only=True)
    
    
    #Train the model
    model.fit(BatchGenerator([x1_train, x2_train], y_train_oh, tps, error_range, batch_size = batch_size_training),
              validation_data = BatchGenerator([x1_val, x2_val], y_val_oh, tps, error_range, batch_size = 25),
              callbacks=[checkpoint, early_stopping], 
              epochs =start+epochs, initial_epoch=start)
    
    model.save('M1_20_model.h5')
    
    print('Training is finished. The trained weights can be found at Data/best_model_weights')
    print('The full model is also saved as Data/M1_20_model.h5')
   
    
if __name__ == '__main__':
    main()

